#include "simulator.h"
#include <algorithm>
#include <thread>

void Duck::quack_and_paint(QPainter& painter) {
    painter.drawRect(x, y, 2, 2);
}

void Duck::move() {
    x += 2;
    y += 2;
}

void RubberDuck::move() {
    x += 2;
}

void Flock::move() {
    std::for_each(quackables.begin(), quackables.end(), [](Quackable* q) { q->move(); });
}

void Flock::quack_and_paint(QPainter& painter) {
    std::for_each(quackables.begin(), quackables.end(), [&](Quackable* q) { q->quack_and_paint(painter); });
}

Simulator::Simulator(int width, int height): width(width), height(height) {
    quackables.push_back(generate_flock());
    quackables.push_back(generate_flock());
    quackables.push_back(new RubberDuck(50, height - 50));
    std::thread([&](){
        while (true) {
            std::this_thread::sleep_for(std::chrono::milliseconds(100));
            std::for_each(quackables.begin(), quackables.end(), [](Quackable* q) { q->move(); });
            emit updated();
        }
    }).detach();
}

Simulator::~Simulator()
{
    // todo: destroy all simulator objects
}

void Simulator::paint(QPainter& painter) {
    std::for_each(quackables.begin(), quackables.end(), [&](Quackable* q) { q->quack_and_paint(painter); });
}

Flock* Simulator::generate_flock() {
    Flock* flock = new Flock();
    flock->add(new MallardDuck(width/2 - 20, height/2));
    flock->add(new MallardDuck(width/2, height/2));
    flock->add(new MallardDuck(width/2 + 20, height/2));
    return flock;
}
