#ifndef SIMULATOR_H
#define SIMULATOR_H

#include <vector>
#include <QObject>
#include <QPainter>

class Quackable {
public:
    virtual ~Quackable() { }
    virtual void move() = 0;
    virtual void quack_and_paint(QPainter& painter) = 0;
};

class Duck: public Quackable {
public:
    Duck(int x, int y): x(x), y(y) { }
    void move();
    void quack_and_paint(QPainter& painter);
protected:
    int x;
    int y;
};

class MallardDuck: public Duck {
public:
    MallardDuck(int x, int y): Duck(x, y) { }
};

class RubberDuck: public Duck {
public:
    RubberDuck(int x, int y): Duck(x, y) { }
    void move();
};

class Flock: public Quackable {
public:
    void add(Quackable* q) { quackables.push_back(q); }
    void move();
    void quack_and_paint(QPainter& painter);
private:
    std::vector<Quackable*> quackables;
};

class Simulator: public QObject
{
    Q_OBJECT
public:
    Simulator(int width, int height);
    ~Simulator();
    void paint(QPainter& painter);
    int get_width() { return width; }
    int get_height() { return height; }
signals:
    void updated();
private:
    int width;
    int height;
    std::vector<Quackable*> quackables;

    Flock* generate_flock();
};

#endif // SIMULATOR_H
